"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../utils");
var util_1 = require("../utils/diff/util");
// This function mutates `linesOfCode` argument.
var applyHunk = function (linesOfCode, _a) {
    var newStart = _a.newStart, changes = _a.changes;
    // Within each hunk, changes are continous, so we can use a sequential algorithm here.
    //
    // When `linesOfCode` is received here, it has already patched by previous hunk,
    // thus the starting line number has changed due to possible unbanlanced deletions and insertions,
    // we should use `newStart` as the first line number of current reduce.
    var patchedLines = changes.reduce(function (_a, _b) {
        var lines = _a[0], cursor = _a[1];
        var content = _b.content, isInsert = _b.isInsert, isDelete = _b.isDelete;
        if (isDelete) {
            lines.splice(cursor, 1);
            return [lines, cursor];
        }
        if (isInsert) {
            lines.splice(cursor, 0, content);
        }
        return [lines, cursor + 1];
    }, [linesOfCode, newStart - 1])[0];
    return patchedLines;
};
var applyDiff = function (oldSource, hunks) {
    // `hunks` must be ordered here.
    var patchedLines = hunks.reduce(applyHunk, oldSource.split('\n'));
    return patchedLines.join('\n');
};
var mapChanges = function (changes, side, toValue) {
    if (!changes.length) {
        return [];
    }
    var computeLineNumber = side === 'old' ? utils_1.computeOldLineNumber : utils_1.computeNewLineNumber;
    var changesByLineNumber = changes.reduce(function (acc, change) {
        acc[computeLineNumber(change)] = change;
        return acc;
    }, {});
    var maxLineNumber = computeLineNumber(util_1.last(changes));
    return Array.from({ length: maxLineNumber }).map(function (_value, i) { return toValue(changesByLineNumber[i + 1]); });
};
var groupChanges = function (hunks) {
    var changes = hunks.reduce(function (acc, hunk) {
        acc.push.apply(acc, hunk.changes);
        return acc;
    }, []);
    return changes.reduce(function (_a, change) {
        var oldChanges = _a[0], newChanges = _a[1];
        if (change.isNormal) {
            oldChanges.push(change);
            newChanges.push(change);
        }
        else if (change.isDelete) {
            oldChanges.push(change);
        }
        else {
            newChanges.push(change);
        }
        return [oldChanges, newChanges];
    }, [[], []]);
};
var toTextPair = function (hunks) {
    var toText = function (change) { return (change ? change.content : ''); };
    var _a = groupChanges(hunks), oldChanges = _a[0], newChanges = _a[1];
    var oldText = mapChanges(oldChanges, 'old', toText).join('\n');
    var newText = mapChanges(newChanges, 'new', toText).join('\n');
    return [oldText, newText];
};
var createRoot = function (children) { return ({ type: 'root', children: children }); };
exports.default = (function (hunks, _a) {
    var highlight = _a.highlight, refractor = _a.refractor, oldSource = _a.oldSource, language = _a.language;
    if (oldSource) {
        var newSource = applyDiff(oldSource, hunks);
        var highlightText = highlight
            ? function (text, language) { return refractor.highlight(text, language); }
            : function (text) { return [{ type: 'text', value: text }]; };
        return [createRoot(highlightText(oldSource, language)), createRoot(highlightText(newSource, language))];
    }
    var textPair = toTextPair(hunks);
    var toTree = highlight
        ? function (text) { return createRoot(refractor.highlight(text, language)); }
        : function (text) { return createRoot([{ type: 'text', value: text }]); };
    return textPair.map(toTree);
});
//# sourceMappingURL=to_token_trees.js.map